#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code shows the functions for STAC collections endpoint
"""
__author__ = "Carmen Tawalika, Jorge Herrera"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"
__maintainer__ = "__mundialis__"

import json
import re

import requests
from werkzeug.exceptions import BadRequest
from actinia_api import URL_PREFIX

from actinia_stac_plugin.core.stac_kvdb_interface import (
    kvdb_actinia_interface,
)
from actinia_stac_plugin.core.common import (
    collectionValidation,
    connectKvdb,
    defaultInstance,
    readStacCollection,
    resolveCollectionURL,
)


def StacCollectionsList():
    connectKvdb()
    stac_inventary = {"collections": []}
    exist = kvdb_actinia_interface.exists("stac_instances")

    if exist:
        instances = kvdb_actinia_interface.read("stac_instances")
        for k, v in instances.items():
            collections = kvdb_actinia_interface.read(k)
            for i, j in collections.items():
                stac = readStacCollection(k, i)
                try:
                    stac = stac.decode("utf8").replace("'", '"')
                except Exception:
                    stac = stac
                # if response is slow (especially with growing collections),
                # it might be an option to use pickle to store json in kvdb
                json_collection = json.loads(stac)
                json_collection["id"] = i
                stac_inventary["collections"].append(json_collection)
    else:
        collections = defaultInstance()
        stac_inventary["defaultStac"] = collections
        kvdb_actinia_interface.create(
            "stac_instances",
            {
                "defaultStac": {
                    "path": "stac.defaultStac.rastercube.<stac_collection_id>"
                }
            },
        )

    return stac_inventary


def addStac2User(jsonparameters):
    """
    Add the STAC Collection to kvdb
        1. Update the Collection to the initial list GET /stac
        2. Store the JSON as a new variable in kvdb
    """
    # Initializing Kvdb
    connectKvdb()

    # Splitting the inputs
    stac_instance_id = jsonparameters["stac_instance_id"]
    stac_root = resolveCollectionURL(jsonparameters["stac_url"])
    stac_json_collection = jsonparameters["collection"]
    stac_collection_id = jsonparameters["stac_collection_id"]

    # Verifying the existence of the instances - Adding the item to the Default List
    list_instances_exist = kvdb_actinia_interface.exists("stac_instances")
    if not list_instances_exist:
        defaultInstance()

    stac_instance_exist = kvdb_actinia_interface.exists(stac_instance_id)

    if not stac_instance_exist:
        raise BadRequest("No Instance name matched")

    if not stac_root:
        raise BadRequest(
            "<%s> is not a valid STAC collection" % jsonparameters["stac_url"]
        )

    if stac_instance_id and stac_root:
        # Caching JSON from the STAC collection
        stac_unique_id = (
            "stac." + stac_instance_id + ".rastercube." + stac_collection_id
        )
        kvdb_actinia_interface.create(
            stac_unique_id, stac_json_collection.content
        )

        defaultJson = kvdb_actinia_interface.read(stac_instance_id)

        defaultJson[stac_unique_id] = {
            "root": stac_root,
            "href": URL_PREFIX[1:] + "/stac/collections/" + stac_unique_id,
        }

        instance_updated = kvdb_actinia_interface.update(
            stac_instance_id, defaultJson
        )

        if instance_updated:
            response = {
                "message": "The STAC Collection has been added successfully",
                "StacCollection": kvdb_actinia_interface.read(
                    stac_instance_id
                ),
            }
        else:
            raise BadRequest(
                "Check the stac_instance_id , stac_url or stac_collection_id given"
            )

        return response


def addStacCollection(parameters):
    """
    The function validate the inputs syntax and STAC validity
    Input:
        - json - JSON array with the Instance ID , Collection ID and STAC URL
    """
    stac_instance_id = "stac_instance_id" in parameters
    stac_root = "stac_url" in parameters
    msg = {}

    if stac_instance_id and stac_root:
        root_validation = collectionValidation(parameters["stac_url"])

        parameters["collection"] = requests.get(parameters["stac_url"])
        parameters["stac_collection_id"] = parameters["collection"].json()[
            "id"
        ]

        collection_validation = re.match(
            "^[a-zA-Z0-9-_]*$", parameters["stac_collection_id"]
        )
        instance_validation = re.match(
            "^[a-zA-Z0-9_]*$", parameters["stac_instance_id"]
        )

        if root_validation and instance_validation and collection_validation:
            return addStac2User(parameters)
        elif not root_validation:
            raise BadRequest(
                "Check the URL provided (Should be a STAC Collection)."
            )
        elif not collection_validation:
            raise BadRequest(
                "Please check the URL provided (Should be a STAC Collection)."
            )
        elif not instance_validation:
            raise BadRequest(
                "Please check the ID given (no spaces or hypens)."
            )

        return msg
    else:
        raise BadRequest("Check the parameters (stac_instance_id,stac_url)")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code shows the transactions valids for STAC collections id endpoints
"""
__author__ = "Carmen Tawalika, Jorge Herrera"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"
__maintainer__ = "__mundialis__"


from actinia_rest_lib.resource_base import ResourceBase
from flask import make_response

from flask_restful_swagger_2 import swagger

from actinia_stac_plugin.core.stac_collection_id import (
    callStacCollection,
    deleteStacCollection,
)

from actinia_stac_plugin.apidocs import stac_collections_id_docs


class StacCollections(ResourceBase):
    """Get the Collections STAC"""

    def __init__(self):
        ResourceBase.__init__(self)

    @swagger.doc(stac_collections_id_docs.staccollection_id_get_docs)
    def get(self, stac_collection_id: str):
        """Get a list of specified Collection."""

        collection_list = callStacCollection(stac_collection_id)

        return make_response(collection_list, 200)

    @swagger.doc(stac_collections_id_docs.staccollection_id_delete_docs)
    def delete(self, stac_collection_id: str):
        """
        This function deletes the STAC collection with the given id.
        Arg:
            - ID - ID/Name given to the STAC Collection you want to delete
        """
        try:
            stac_instance_id = stac_collection_id.split(".")[1]
        except Exception:
            message = (
                "stac_collection_id does not match the pattern "
                "stac.<stac_instance_id>.rastercube.<stac_collection_id"
            )

            return make_response(message, 400)

        deleted_stac = deleteStacCollection(
            stac_instance_id, stac_collection_id
        )

        return make_response(deleted_stac, 200)



# -*- coding: utf-8 -*-
import importlib.metadata

try:
    # Change here if project is renamed and does not equal the package name
    dist_name = __name__
    __version__ = importlib.metadata.version(dist_name)
except Exception:
    __version__ = "unknown"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.


Documentation for STAC module api endpoints
"""

__author__ = "Jorge Herrera"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"

staccatalog_get_docs = {
    "tags": ["STAC"],
    "description": "Get a list of STAC result-catalog. "
    "Minimum required user role: user.",
    "parameters": [],
    "responses": {
        "200": {"description": "This response returns a list of STAC catalogs"}
    },
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code shows the functions for STAC instance endpoint
"""
__author__ = "Carmen Tawalika, Jorge Herrera"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"
__maintainer__ = "__mundialis__"

import re
from werkzeug.exceptions import BadRequest

from actinia_stac_plugin.core.stac_kvdb_interface import (
    kvdb_actinia_interface,
)
from actinia_stac_plugin.core.common import connectKvdb, defaultInstance


def createStacItemList():
    connectKvdb()
    exist = kvdb_actinia_interface.exists("stac_instances")

    if not exist:
        defaultInstance()
        kvdb_actinia_interface.create(
            "stac_instances",
            {
                "defaultStac": {
                    "path": "stac.defaultStac.rastercube.<stac_collection_id>"
                }
            },
        )

    instances = kvdb_actinia_interface.read("stac_instances")

    return instances


def addInstance2User(jsonparameters):
    """
    Add the STAC Collection to kvdb
        1. Update the Collection to the initial list GET /stac
        2. Store the JSON as a new variable in kvdb
    """
    # Initializing Kvdb
    connectKvdb()

    # Splitting the inputs
    stac_instance_id = jsonparameters["stac_instance_id"]
    # Verifying the existence of the instances - Adding the item to the Default List
    list_instances_exist = kvdb_actinia_interface.exists("stac_instances")
    if not list_instances_exist:
        defaultInstance()

    stac_instance_exist = kvdb_actinia_interface.exists(stac_instance_id)

    if not stac_instance_exist:
        kvdb_actinia_interface.create(stac_instance_id, {})

        instances_list = kvdb_actinia_interface.read("stac_instances")
        instances_list[stac_instance_id] = {
            "path": "stac."
            + stac_instance_id
            + ".rastercube.<stac_collection_id>"
        }

        list_of_instances_updated = kvdb_actinia_interface.update(
            "stac_instances", instances_list
        )

        if list_of_instances_updated:
            response = kvdb_actinia_interface.read(stac_instance_id)
        else:
            raise BadRequest("Check the stac_instance_id given")

        return response


def addInstance(parameters):
    """
    The function validate the inputs syntax and STAC validity
    Input:
        - parameters - JSON array with the Instance ID
    """
    stac_instance_id = "stac_instance_id" in parameters
    msg = {}

    if stac_instance_id:
        instance_validation = re.match(
            "^[a-zA-Z0-9_]*$", parameters["stac_instance_id"]
        )

        if instance_validation:
            return addInstance2User(parameters)
        elif not instance_validation:
            raise BadRequest(
                "Please check the ID given (no spaces or undercore characters)."
            )

        return msg
    else:
        raise BadRequest("Check the parameters (stac_instance_id)")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.


Logging interface
"""

__license__ = "GPLv3"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2019-2021, mundialis"
__maintainer__ = "Carmen Tawalika"

# pylint: disable=E1101
# E1101: Instance of 'RootLogger' has no 'loggerDict' member (no-member) - Make no sense

import logging
from datetime import datetime

from colorlog import ColoredFormatter
from pythonjsonlogger import jsonlogger

from actinia_stac_plugin.resources.config import LOGCONFIG

log = logging.getLogger("actinia-stac-plugin")
werkzeuglog = logging.getLogger("werkzeug")
gunicornlog = logging.getLogger("gunicorn")


def setLogFormat(veto=None):
    logformat = ""
    if LOGCONFIG.type == "json" and not veto:
        logformat = CustomJsonFormatter(
            "%(time) %(level) %(component)"
            "%(module) %(message) %(pathname)"
            "%(lineno) %(processName)"
            "%(threadName)"
        )
    else:
        logformat = ColoredFormatter(
            "%(log_color)s[%(asctime)s] %(levelname)-10s: %(name)s.%(module)-"
            "10s -%(message)s [in %(pathname)s:%(lineno)d]%(reset)s"
        )
    return logformat


def setLogHandler(logger, type, format):
    if type == "stdout":
        handler = logging.StreamHandler()
    elif type == "file":
        # For readability, json is never written to file
        handler = logging.FileHandler(LOGCONFIG.logfile)
    handler.setFormatter(format)
    logger.addHandler(handler)


class CustomJsonFormatter(jsonlogger.JsonFormatter):
    def add_fields(self, log_record, record, message_dict):
        super(CustomJsonFormatter, self).add_fields(
            log_record, record, message_dict
        )

        # (Pdb) dir(record)
        # ... 'args', 'created', 'exc_info', 'exc_text', 'filename', 'funcName'
        # ,'getMessage', 'levelname', 'levelno', 'lineno', 'message', 'module',
        # 'msecs', 'msg', 'name', 'pathname', 'process', 'processName',
        # 'relativeCreated', 'stack_info', 'thread', 'threadName']

        now = datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S.%fZ")
        log_record["time"] = now
        log_record["level"] = record.levelname
        log_record["component"] = record.name


def createLogger():
    # create logger, set level and define format
    log.setLevel(getattr(logging, LOGCONFIG.level))
    fileformat = setLogFormat("veto")
    stdoutformat = setLogFormat()
    setLogHandler(log, "file", fileformat)
    setLogHandler(log, "stdout", stdoutformat)


def createWerkzeugLogger():
    werkzeuglog.setLevel(getattr(logging, LOGCONFIG.level))
    fileformat = setLogFormat("veto")
    stdoutformat = setLogFormat()
    setLogHandler(werkzeuglog, "file", fileformat)
    setLogHandler(werkzeuglog, "stdout", stdoutformat)


def createGunicornLogger():
    gunicornlog.setLevel(getattr(logging, LOGCONFIG.level))
    fileformat = setLogFormat("veto")
    stdoutformat = setLogFormat()
    setLogHandler(gunicornlog, "file", fileformat)
    setLogHandler(gunicornlog, "stdout", stdoutformat)
    # gunicorn already has a lot of children logger, e.g gunicorn.http,
    # gunicorn.access. These lines deactivate their default handlers.
    for name in logging.root.manager.loggerDict:
        if "gunicorn." in name:
            logging.getLogger(name).propagate = True
            logging.getLogger(name).handlers = []


createLogger()
createWerkzeugLogger()
createGunicornLogger()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.


Documentation for STAC module api endpoints
"""

__author__ = "Jorge Herrera"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"


stacinstance_id_get_docs = {
    "tags": ["STAC"],
    "description": "Get the instance with the id given "
    "Minimum required user role: user.",
    "parameters": [
        {
            "in": "path",
            "name": "stac_instance_id",
            "type": "string",
            "description": "the instance id of the collection to be obtained",
            "required": True,
        }
    ],
    "responses": {
        "200": {
            "description": "This response returns a instance with its collections"
        },
        "400": {
            "description": "This response returns a detail error message",
            "schema": {
                "type": "object",
                "properties": {
                    "message": {
                        "type": "string",
                        "description": "detailed message",
                        "example": "stac instance ID does not match with the instences stored",
                    }
                },
            },
        },
    },
}

stacinstance_id_delete_docs = {
    "tags": ["STAC"],
    "description": "Delete an instance",
    "parameters": [
        {
            "in": "path",
            "name": "stac_instance_id",
            "type": "string",
            "description": "the instance id to be deleted (All collections will be removed)",
            "required": True,
        }
    ],
    "responses": {
        "200": {
            "description": "This response returns a message with the "
            + "instance and the STAC collections deleted"
        },
        "400": {
            "description": "This response returns a detail error message",
            "schema": {
                "type": "object",
                "properties": {
                    "message": {
                        "type": "string",
                        "description": "detailed message",
                        "example": "Something went wrong please check that the element is well typed",
                    }
                },
            },
        },
    },
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code shows the functions for STAC endpoint
"""
__author__ = "Carmen Tawalika, Jorge Herrera"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"
__maintainer__ = "__mundialis__"


from actinia_stac_plugin.core.stac_kvdb_interface import (
    kvdb_actinia_interface,
)
from actinia_stac_plugin.core.common import connectKvdb, defaultInstance


def createStacItemList():
    connectKvdb()
    exist = kvdb_actinia_interface.exists("stac_instances")

    if not exist:
        defaultInstance()
        kvdb_actinia_interface.create(
            "stac_instances",
            {
                "defaultStac": {
                    "path": "stac.defaultStac.rastercube.<stac_collection_id>"
                }
            },
        )

    instances = kvdb_actinia_interface.read("stac_instances")

    return instances



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.


Test code for STAC module api endpoints
"""
__author__ = "Jorge Herrera"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"

import json

from testsuite import ActiniaTestCase
from actinia_api import URL_PREFIX


class StacInstancesEndpointTest(ActiniaTestCase):
    def test_b_get_instances(self):
        """Test if get instances responds"""
        resp = self.app.get(
            f"{URL_PREFIX}/stac/instances", headers=self.user_auth_header
        )

        assert resp.status_code == 200
        assert hasattr(resp, "json")

    def test_c_post_error_instances(self):
        """Test if add a new instance responds"""

        respStatusCode = 400

        instance_add_body = {"stac_instance": ""}

        resp = self.app.post(
            f"{URL_PREFIX}/stac/instances",
            headers=self.user_auth_header,
            data=json.dumps(instance_add_body),
            content_type="application/json",
        )

        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

    def test_d_post_instances(self):
        """Test if add a new instance responds"""

        respStatusCode = 200

        instance_add_body = {"stac_instance_id": "STACtestinstance"}

        resp = self.app.post(
            f"{URL_PREFIX}/stac/instances",
            headers=self.user_auth_header,
            data=json.dumps(instance_add_body),
            content_type="application/json",
        )

        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

    def test_e_get_instance_id(self):
        """Test if get an instance responds"""

        stac_instance_id = "STACtestinstance"
        resp = self.app.get(
            f"{URL_PREFIX}/stac/instances/" + stac_instance_id,
            headers=self.user_auth_header,
        )
        assert resp.status_code == 200
        assert hasattr(resp, "json")

    def test_f_get_error_instance_id(self):
        """Test if get an instance responds"""

        stac_instance_id = "STAC-NoExist-"
        resp = self.app.get(
            f"{URL_PREFIX}/stac/instances/" + stac_instance_id,
            headers=self.user_auth_header,
        )
        assert resp.status_code == 400
        assert hasattr(resp, "json")

    def test_g_delete_instance_id(self):
        """Test if delete an instance responds"""

        stac_instance_id = "STACtestinstance"
        resp = self.app.delete(
            f"{URL_PREFIX}/stac/instances/" + stac_instance_id,
            headers=self.user_auth_header,
        )

        assert resp.status_code == 200
        assert hasattr(resp, "json")

    def test_h_delete_error_instance_id(self):
        """Test if delete an instance responds"""

        stac_instance_id = "STAC-NoExist-"
        resp = self.app.delete(
            f"{URL_PREFIX}/stac/instances/" + stac_instance_id,
            headers=self.user_auth_header,
        )

        assert resp.status_code == 400
        assert hasattr(resp, "json")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code shows the functions for STAC collection id endpoint
"""
__author__ = "Carmen Tawalika, Jorge Herrera"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"
__maintainer__ = "__mundialis__"


import json
from werkzeug.exceptions import BadRequest

from actinia_stac_plugin.core.stac_kvdb_interface import (
    kvdb_actinia_interface,
)
from actinia_stac_plugin.core.common import readStacCollection, connectKvdb


def callStacCollection(stac_collection_id: str):
    try:
        instance_id = stac_collection_id.split(".")[1]
        stac = readStacCollection(instance_id, stac_collection_id)
        resp = json.loads(stac)
        # overwrite original ID with generated ID
        resp["id"] = stac_collection_id
    except Exception:
        raise BadRequest("Please check the collection id provided")

    return resp


def deleteStacCollection(stac_instance_id: str, stac_collection_id: str):
    connectKvdb()

    try:
        stac_instance = kvdb_actinia_interface.read(stac_instance_id)
        del stac_instance[stac_collection_id]
        kvdb_actinia_interface.update(stac_instance_id, stac_instance)
        if kvdb_actinia_interface.exists(stac_collection_id):
            kvdb_actinia_interface.delete(stac_collection_id)
    except Exception:
        raise BadRequest(
            "Please check that the parameters given are well typed and exist"
        )

    return kvdb_actinia_interface.read(stac_instance_id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code shows the functions for STAC item endpoint
"""
__author__ = "Carmen Tawalika, Jorge Herrera"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"
__maintainer__ = "__mundialis__"

from werkzeug.exceptions import BadRequest

from actinia_stac_plugin.core.stac_kvdb_interface import (
    kvdb_actinia_interface,
)
from actinia_stac_plugin.core.common import connectKvdb


def getStacItem(item: str, item_id: str):
    connectKvdb()
    exist = kvdb_actinia_interface.exists(item)

    if not exist:
        raise BadRequest("No Item found with the provided parameters")

    item = kvdb_actinia_interface.read(item)

    return item



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.


Test code for STAC module api endpoints
"""
__author__ = "Jorge Herrera"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"

from testsuite import ActiniaTestCase
from actinia_api import URL_PREFIX


class StacEndpointTest(ActiniaTestCase):
    def test_a_app_instances(self):
        """Test if app responds"""
        resp = self.app.get(
            f"{URL_PREFIX}/stac", headers=self.user_auth_header
        )

        assert resp.status_code == 200
        assert hasattr(resp, "json")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.


Configuration file
"""

__author__ = "Carmen Tawalika"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"


import configparser
from pathlib import Path

# config can be overwritten by mounting *.ini files into folders inside
# the config folder.
DEFAULT_CONFIG_PATH = "config"
CONFIG_FILES = [
    str(f) for f in Path(DEFAULT_CONFIG_PATH).glob("**/*.ini") if f.is_file()
]
GENERATED_CONFIG = DEFAULT_CONFIG_PATH + "/actinia-stac-plugin.cfg"


class LOGCONFIG:
    """Default config for logging"""

    logfile = "actinia-stac-plugin.log"
    level = "DEBUG"
    type = "stdout"


class Configfile:
    def __init__(self):
        """
        This class will overwrite the config classes above when config files
        named DEFAULT_CONFIG_PATH/**/*.ini exist.
        On first import of the module it is initialized.
        """

        config = configparser.ConfigParser()
        config.read(CONFIG_FILES)

        if len(config) <= 1:
            print("Could not find any config file, using default values.")
            return
        print("Loading config files: " + str(CONFIG_FILES) + " ...")

        with open(GENERATED_CONFIG, "w") as configfile:
            config.write(configfile)
        print("Configuration written to " + GENERATED_CONFIG)

        # LOGGING
        if config.has_section("LOGCONFIG"):
            if config.has_option("LOGCONFIG", "logfile"):
                LOGCONFIG.logfile = config.get("LOGCONFIG", "logfile")
            if config.has_option("LOGCONFIG", "level"):
                LOGCONFIG.level = config.get("LOGCONFIG", "level")
            if config.has_option("LOGCONFIG", "type"):
                LOGCONFIG.type = config.get("LOGCONFIG", "type")


init = Configfile()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.


Add endpoints to flask app with endpoint definitions and routes
"""

__author__ = "Carmen Tawalika"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"


# from flask import current_app, send_from_directory
# import werkzeug

# from actinia_module_plugin.resources.logging import log

from actinia_stac_plugin.api.stac import Stac
from actinia_stac_plugin.api.stac_collections import StacCollectionList
from actinia_stac_plugin.api.stac_collection_id import StacCollections
from actinia_stac_plugin.api.stac_instances import StacInstanceList
from actinia_stac_plugin.api.stac_instance_id import StacInstances
from actinia_stac_plugin.api.stac_catalogs import StacCatalogList
from actinia_stac_plugin.api.stac_items import StacItems


def create_endpoints(flask_api):
    # app = flask_api.app
    apidoc = flask_api

    # @app.route('/')
    # def index():
    #     try:
    #         # flask cannot reach out of current_app (which is actinia_core)
    #         return current_app.send_static_file('index.html')
    #     except werkzeug.exceptions.NotFound:
    #         log.debug('No index.html found. Serving backup.')
    #         return ("""<h1 style='color:red'>actinia</h1>
    #             <a href="swagger.json">API docs</a>""")
    #
    # @app.route('/<path:filename>')
    # def static_content(filename):
    #     # WARNING: all content from folder "static" will be accessible!
    #     return send_from_directory(app.static_folder, filename)

    apidoc.add_resource(Stac, "/stac")
    apidoc.add_resource(StacCollectionList, "/stac/collections")
    apidoc.add_resource(
        StacCollections, "/stac/collections/<string:stac_collection_id>"
    )
    apidoc.add_resource(StacInstanceList, "/stac/instances")
    apidoc.add_resource(
        StacInstances, "/stac/instances/<string:stac_instance_id>"
    )
    apidoc.add_resource(StacCatalogList, "/stac/catalogs/catalog.json")
    apidoc.add_resource(
        StacItems, "/stac/catalogs/<string:item>/<string:item_id>.json"
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code shows the functional elements in common for STAC endpoints
"""
__author__ = "Carmen Tawalika, Jorge Herrera"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"
__maintainer__ = "__mundialis__"

import requests
from werkzeug.exceptions import BadRequest
from actinia_core.core.common.config import Configuration
from actinia_api import URL_PREFIX
from stac_validator import stac_validator

from actinia_stac_plugin.core.stac_kvdb_interface import (
    kvdb_actinia_interface,
)


def connectKvdb():
    """This method initializes the connection with a kvdb."""
    conf = Configuration()
    try:
        conf.read()
    except Exception:
        print("Error reading configuration")

    server = conf.KVDB_SERVER_URL
    port = conf.KVDB_SERVER_PORT
    if conf.KVDB_SERVER_PW:
        kvdb_password = conf.KVDB_SERVER_PW
    else:
        kvdb_password = None

    kvdb_actinia_interface.connect(
        host=server, port=port, password=kvdb_password
    )

    return kvdb_actinia_interface


def defaultInstance():
    connectKvdb()
    exist = kvdb_actinia_interface.exists("defaultStac")

    defaultStac = {
        "stac.defaultStac.rastercube.landsat-8-l1-c1": {
            "root": "https://earth-search.aws.element84.com/v0/collections/landsat-8-l1-c1",
            "href": f"{URL_PREFIX}/stac/collections/stac.defaultStac.rastercube.landsat-8-l1-c1",
        },
        "stac.defaultStac.rastercube.sentinel-s2-l2a": {
            "root": "https://earth-search.aws.element84.com/v0/collections/sentinel-s2-l2a",
            "href": f"{URL_PREFIX}/stac/collections/stac.defaultStac.rastercube.sentinel-s2-l2a",
        },
    }

    if exist:
        return kvdb_actinia_interface.read("defaultStac")
    else:
        kvdb_actinia_interface.create("defaultStac", defaultStac)
        return kvdb_actinia_interface.read("defaultStac")


def readStacCollection(stac_instance_id: str, stac_collection_id: str):
    connectKvdb()
    try:
        if kvdb_actinia_interface.exists(stac_collection_id):
            stac = kvdb_actinia_interface.read(stac_collection_id)
        else:
            stac_dict = kvdb_actinia_interface.read(stac_instance_id)
            stac_root_url = stac_dict[stac_collection_id]["root"]
            response = requests.get(stac_root_url)
            stac = response.content
    except Exception:
        raise BadRequest(
            "Something went wrong, please check the collection and catalog to retrieved"
        )

    return stac


def collectionValidation(url: str) -> bool:
    """
    Verify that the URL provided belong to a STAC endpoint
    """
    stac = stac_validator.StacValidate(url)
    if stac.run() is False:
        print("This <%s> is not a valid STAC collection" % stac)
        return False
    if "valid_stac" not in stac.message[0].keys():
        print("This <%s> is not a valid STAC collection" % stac)
        return False
    if stac.message[0]["valid_stac"] is False:
        print("This <%s> is not a valid STAC collection" % stac)
        return False
    if "asset_type" not in stac.message[0].keys():
        print("This <%s> is not a valid STAC collection" % stac)
        return False
    if stac.message[0]["asset_type"] != "COLLECTION":
        print("This <%s> is not a valid STAC collection" % stac)
        return False

    return True


def resolveCollectionURL(url):
    stac = stac_validator.StacValidate(url)
    if stac.run() is False:
        print("This <%s> is not a valid STAC collection" % stac)
        return None
    if "valid_stac" not in stac.message[0].keys():
        print("This <%s> is not a valid STAC collection" % stac)
        return None
    if stac.message[0]["valid_stac"] is False:
        print("This <%s> is not a valid STAC collection" % stac)
        return None
    if "asset_type" not in stac.message[0].keys():
        print("This <%s> is not a valid STAC collection" % stac)
        return None
    if stac.message[0]["asset_type"] != "COLLECTION":
        print("This <%s> is not a valid STAC collection" % stac)
        return None

    return url



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

Kvdb interface
This file is not yet fitted to STAC plugin usability,
however, the file will be adjusted in further steps of the development.
"""
__author__ = "Carmen Tawalika, Jorge Herrera"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"
__maintainer__ = "__mundialis__"


import pickle

from actinia_core.core.common.kvdb_base import KvdbBaseInterface


class KvdbActiniaInterface(KvdbBaseInterface):
    """
    The Kvdb actinia_template database interface
    A single actinia_template is stored as Hash with:
        - actinia_template id aka actinia_template name that must be unique
        - actinia-actinia_template dictionary
    In addition is the actinia_template_id saved in a hash that contains all
    actinia_template ids.
    """

    # We use two databases:
    # The actinia_template ID and the actinia_template name database
    # The actinia_template ID and actinia_template name databases are hashes
    actinia_template_id_hash_prefix = "ACTINIA-STAC-ID-PREFIX::"
    actinia_template_id_db = "ACTINIA-STAC-ID-DATABASE"

    def __init__(self):
        KvdbBaseInterface.__init__(self)

    def create(self, actinia_id, actinia_template):
        """
        Add an actinia_stac to the actinia_stac database
        Args:
            actinia_stac (dict): A dictionary of permissions
        Returns:
            bool:
            True is success, False if actinia_stac is already in database
        """
        actinia_template_id = actinia_id

        keyname = self.actinia_template_id_hash_prefix + actinia_template_id
        exists = self.kvdb_server.exists(keyname)
        if exists == 1 or exists is True:
            return False

        actinia_template_bytes = pickle.dumps(actinia_template)
        mapping = {
            "actinia_template_id": actinia_template_id,
            "actinia_template": actinia_template_bytes,
        }

        lock = self.kvdb_server.lock(
            name="add_actinia_template_lock", timeout=1
        )
        lock.acquire()
        # First add the actinia_template-id to the actinia_template id database
        self.kvdb_server.hset(
            self.actinia_template_id_db,
            actinia_template_id,
            actinia_template_id,
        )

        self.kvdb_server.hset(
            self.actinia_template_id_hash_prefix + actinia_template_id,
            mapping=mapping,
        )
        lock.release()

        return True

    def read(self, actinia_template_id):
        """Return the actinia_template
        HGET actinia_template-id actinia_template_group
        Args:
            actinia_template_id: The actinia_template id
        Returns:
             str:
             The actinia_template group
        """

        try:
            actinia_template = pickle.loads(
                self.kvdb_server.hget(
                    self.actinia_template_id_hash_prefix + actinia_template_id,
                    "actinia_template",
                )
            )
        except Exception:
            return False

        return actinia_template

    def update(self, actinia_template_id, actinia_template):
        """Update the actinia_template.
        Renaming an entry is not allowed, only existing entries with the
        same actinia_template_id can be updated.
        Args:
            actinia_template_id (str): The actinia_template id
            actinia_template (dict): The actinia_template as dictionary
        Returns:
            bool:
            True is success, False if actinia_template is not in the database
        """
        keyname = self.actinia_template_id_hash_prefix + actinia_template_id
        exists = self.kvdb_server.exists(keyname)
        if exists == 0 or exists is False:
            return False

        actinia_template_bytes = pickle.dumps(actinia_template)
        mapping = {
            "actinia_template_id": actinia_template_id,
            "actinia_template": actinia_template_bytes,
        }

        lock = self.kvdb_server.lock(
            name="update_actinia_template_lock", timeout=1
        )
        lock.acquire()

        self.kvdb_server.hset(
            self.actinia_template_id_hash_prefix + actinia_template_id,
            mapping=mapping,
        )

        lock.release()

        return True

    def delete(self, actinia_template_id):
        """Remove an actinia_template id from the database
        Args:
            actinia_template_id (str): The actinia_template id
        Returns:
            bool:
            True is actinia_template exists, False otherwise
        """
        exists = self.exists(actinia_template_id)
        if exists == 0 or exists is False:
            return False

        lock = self.kvdb_server.lock(
            name="delete_actinia_template_lock", timeout=1
        )
        lock.acquire()
        # Delete the entry from the actinia_template id database
        self.kvdb_server.hdel(self.actinia_template_id_db, actinia_template_id)
        # Delete the actual actinia_template entry
        self.kvdb_server.delete(
            self.actinia_template_id_hash_prefix + actinia_template_id
        )
        lock.release()

        return True

    def list_all_ids(self):
        """
        List all actinia_template id's that are in the database
        HKEYS on the actinia_template id database
        Returns:
            list:
            A list of all actinia_template ids in the database
        """
        values = []
        list = self.kvdb_server.hkeys(self.actinia_template_id_db)
        for entry in list:
            entry = entry.decode()
            values.append(entry)

        return values

    def exists(self, actinia_template_id):
        """Check if the actinia_template is in the database
        Args:
            actinia_template_id (str): The actinia_template id
        Returns:
            bool:
            True is actinia_template exists, False otherwise
        """
        return self.kvdb_server.exists(
            self.actinia_template_id_hash_prefix + actinia_template_id
        )


# Create the Kvdb interface instance
kvdb_actinia_interface = KvdbActiniaInterface()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.


Application entrypoint. Creates Flask app and swagger docs, adds endpoints
"""

__author__ = "Carmen Tawalika"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"


from flask import Flask
from flask_cors import CORS
from flask_restful_swagger_2 import Api

from actinia_stac_plugin.resources.log import log

app = Flask(__name__)
CORS(app)

apidoc = Api(
    app,
    title="actinia-stac-plugin",
    api_spec_url="/latest/api/swagger",
    schemes=["https", "http"],
    consumes=["application/json"],
    description="""STAC.
                   """,
)

if __name__ == "__main__":
    # call this for development only with
    # `python -m actinia_stac_plugin.main`
    log.debug("starting app in development mode...")
    app.run(debug=False, use_reloader=False)
    # for production environent use application in wsgy.py



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code shows the transactions valids for STAC instances endpoint
"""
__author__ = "Carmen Tawalika, Jorge Herrera"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"
__maintainer__ = "__mundialis__"


from actinia_rest_lib.resource_base import ResourceBase
from flask import make_response, request

from flask_restful_swagger_2 import swagger

from actinia_stac_plugin.core.stac_instances import (
    addInstance,
    createStacItemList,
)

from actinia_stac_plugin.apidocs import stac_instances_docs


class StacInstanceList(ResourceBase):
    def __init__(self):
        ResourceBase.__init__(self)

    @swagger.doc(stac_instances_docs.stacinstances_get_docs)
    def get(self):
        """Get a list of all instances."""
        instances_list = createStacItemList()

        return make_response(instances_list, 200)

    @swagger.doc(stac_instances_docs.stacinstances_post_docs)
    def post(self):
        """
        Add a new stac to the user collection
        """

        parameters = request.get_json(force=True)
        new_stac = addInstance(parameters)

        message = {
            "message": "The Instance has been added successfully",
            "StacInstance": new_stac,
        }

        return make_response(message, 200)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code shows the functions for STAC instance id endpoint
"""
__author__ = "Carmen Tawalika, Jorge Herrera"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"
__maintainer__ = "__mundialis__"

from werkzeug.exceptions import BadRequest

from actinia_stac_plugin.core.stac_kvdb_interface import (
    kvdb_actinia_interface,
)
from actinia_stac_plugin.core.common import connectKvdb


def getInstance(stac_instance_id):
    connectKvdb()
    exist = kvdb_actinia_interface.exists(stac_instance_id)

    if not exist:
        raise BadRequest(
            "stac instance ID does not match with the instences stored"
        )

    return kvdb_actinia_interface.read(stac_instance_id)


def deleteStacInstance(stac_instance_id):
    connectKvdb()
    try:
        instance = kvdb_actinia_interface.read(stac_instance_id)
        for i in instance.keys():
            kvdb_actinia_interface.delete(i)
        kvdb_actinia_interface.delete(stac_instance_id)
        instances = kvdb_actinia_interface.read("stac_instances")
        del instances[stac_instance_id]
        kvdb_actinia_interface.update("stac_instances", instances)
    except Exception:
        raise BadRequest(
            "Something went wrong please that the element is well typed "
            + stac_instance_id
        )
    return stac_instance_id



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.


Test code for STAC module api endpoints
"""
__author__ = "Jorge Herrera"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"


import base64
import unittest
from typing import Dict, List

import pwgen
from actinia_core.endpoints import create_endpoints
from actinia_core.core.common import kvdb_interface
from actinia_core.core.common.app import flask_app
from actinia_core.core.common.config import global_config
from actinia_core.core.common.user import ActiniaUser
from werkzeug.datastructures import Headers


# actinia-stac-plugin endpoints are included as defined in actinia_core
# config
create_endpoints()


class ActiniaTestCase(unittest.TestCase):
    # guest = None
    # admin = None
    # superadmin = None
    user = None
    auth_header: Dict[str, Headers] = {}
    users_list: List[str] = []

    def setUp(self):
        """Overwrites method setUp from unittest.TestCase class"""

        self.app_context = flask_app.app_context()
        self.app_context.push()
        # from http://flask.pocoo.org/docs/0.12/api/#flask.Flask.test_client:
        # Note that if you are testing for assertions or exceptions in your
        # application code, you must set app.testing = True in order for the
        # exceptions to propagate to the test client.  Otherwise, the exception
        # will be handled by the application (not visible to the test client)
        # and the only indication of an AssertionError or other exception will
        # be a 500 status code response to the test client.
        flask_app.testing = True
        self.app = flask_app.test_client()

        # Start and connect the kvdb interface
        kvdb_args = (
            global_config.KVDB_SERVER_URL,
            global_config.KVDB_SERVER_PORT,
        )
        if (
            global_config.KVDB_SERVER_PW
            and global_config.KVDB_SERVER_PW is not None
        ):
            kvdb_args = (*kvdb_args, global_config.KVDB_SERVER_PW)
        kvdb_interface.connect(*kvdb_args)

        # create test user for roles user (more to come)
        accessible_datasets = {
            "nc_spm_08": ["PERMANENT", "user1", "modis_lst"]
        }
        password = pwgen.pwgen()
        self.user_id, self.user_group, self.user_auth_header = self.createUser(
            name="user",
            role="user",
            password=password,
            process_num_limit=3,
            process_time_limit=4,
            accessible_datasets=accessible_datasets,
        )

        # # create process queue
        # from actinia_core.core.common.process_queue import \
        #    create_process_queue
        # create_process_queue(config=global_config)

    def tearDown(self):
        """Overwrites method tearDown from unittest.TestCase class"""

        self.app_context.pop()

        # remove test user; disconnect kvdb
        for user in self.users_list:
            user.delete()
        kvdb_interface.disconnect()

    def createUser(
        self,
        name="guest",
        role="guest",
        group="group",
        password="abcdefgh",
        accessible_datasets=None,
        process_num_limit=1000,
        process_time_limit=6000,
    ):
        auth = bytes("%s:%s" % (name, password), "utf-8")

        # We need to create an HTML basic authorization header
        self.auth_header[role] = Headers()
        self.auth_header[role].add(
            "Authorization", "Basic " + base64.b64encode(auth).decode()
        )

        # Make sure the user database is empty
        user = ActiniaUser(name)
        if user.exists():
            user.delete()
        # Create a user in the database
        user = ActiniaUser.create_user(
            name,
            group,
            password,
            user_role=role,
            accessible_datasets=accessible_datasets,
            process_num_limit=process_num_limit,
            process_time_limit=process_time_limit,
        )
        user.add_accessible_modules(["uname", "sleep"])
        self.users_list.append(user)

        return name, group, self.auth_header[role]



#!/usr/bin/env python
"""Copyright (c) 2022-2025 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

Setup file for actinia_stac_plugin.
"""

__author__ = "Jorge Herrera, Carmen Tawalika"
__copyright__ = "2021-2025 mundialis GmbH & Co. KG"
__license__ = "GPLv3"

from setuptools import setup

if __name__ == "__main__":
    setup()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 mundialis GmbH & Co. KG

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.


Documentation for STAC module api endpoints
"""

__author__ = "Jorge Herrera"
__copyright__ = "2018-2022 mundialis GmbH & Co. KG"
__license__ = "GPLv3"


staccollection_get_docs = {
    "tags": ["STAC"],
    "description": "Get a list of STAC collections"
    "Minimum required user role: user.",
    "parameters": [],
    "responses": {
        "200": {
            "description": "This response returns a list of STAC collections"
        }
    },
}

staccollection_post_docs = {
    "tags": ["STAC"],
    "description": "Add a new STAC collection to the user instance",
    "parameters": [
        {
            "in": "body",
            "name": "body",
            "type": "object",
            "description": "the instance id where the collection will be stored",
            "schema": {
                "type": "object",
                "properties": {
                    "stac_instance_id": {
                        "type": "string",
                        "description": "instance id where the collection will be stored",
                        "example": "ProjectInstance",
                    },
                    "stac_url": {
                        "type": "string",
                        "description": "url where the collection is located",
                        "example": "https://earth-search.aws.element84.com/v0/collections/sentinel-s2-l2a",
                    },
                },
            },
            "required": True,
        }
    ],
    "responses": {
        "200": {
            "description": "This response returns a message with the STAC collection successfully added"
        },
        "400": {
            "description": "This response returns a detail error message",
            "schema": {
                "type": "object",
                "properties": {
                    "message": {
                        "type": "string",
                        "description": "detailed message",
                        "example": "Please check the URL provided (Should be a STAC Collection)",
                    }
                },
            },
        },
    },
}
